<?php declare(strict_types=1);

namespace Everyware\Plugin\CacheInvalidator;

use Everyware\Plugin\CacheInvalidator\LedgeInvalidator;
use Everyware\Plugin\CacheInvalidator\PageSettings;

/**
 * Class Lcc
 * @package Everyware\Plugin\CacheInvalidator
 */
class Lcc
{

    private $baseUrl;
    private $username;
    private $password;

    /**
     * Lcc constructor.
     */
    public function __construct()
    {
        $this->baseUrl = PageSettings::getQueryStreamerUrl();
        $this->username = PageSettings::getQueryStreamerUsername();
        $this->password = PageSettings::getQueryStreamerPassword();
    }

    /**
     * Connect to LCC.
     *
     * @return bool
     */
    public function connect(): bool
    {
        return true;
    }

    /**
     * Register a listener.
     *
     * @param $query
     *
     * @return string or false
     */
    public function registerListener($query)
    {
        $query = trim($query . ' ' . PageSettings::getQuerySuffix());
        $json_data = [
            'query' => [
                'query_string' => [
                    'query' => $query
                ]
            ],
            'destination' => [
                'type' => 'cache',
                'url' => plugins_url('src/push.php', dirname(__FILE__) )
            ],
            'meta' => [
                'msg' => 'Automated query stream from Everyware ew-cache-invalidator plugin.'
            ],
            'config' => [
                'notifyOnNoLongerMatchingQuery' => true
            ]
        ];

        $response = wp_remote_post($this->baseUrl . 'streams', [
            'headers' => [
                'Authorization' => 'Basic ' . base64_encode($this->username . ':' . $this->password),
                'Content-Type' => 'application/json'
            ],
            'body' => json_encode($json_data)
        ]);

        if ( ! is_wp_error($response) && $response['response']['code'] === 200) {

            $response_json = json_decode($response['body']);

            if (isset($response_json->streamId)) {
                return $response_json->streamId;
            }
        }

        return false;
    }

    /**
     * Deregister a listener.
     *
     * @param $streamId
     *
     * @return bool
     */
    public function deregisterListener($streamId): bool
    {
        $response = wp_remote_request($this->baseUrl . 'streams/' . $streamId, [
            'method' => 'DELETE',
            'headers' => [
                'Authorization' => 'Basic ' . base64_encode($this->username . ':' . $this->password)
            ]
        ]);

        return ! is_wp_error($response) && $response['response']['code'] === 200;
    }

    /**
     * Get connection status.
     *
     * @return bool
     */
    public function getStatus(): bool
    {
        $response = wp_remote_get($this->baseUrl . 'streams?url=test&type=cache', [
            'headers' => [
                'Authorization' => 'Basic ' . base64_encode($this->username . ':' . $this->password)
            ]
        ]);

        return ! is_wp_error($response) && $response['response']['code'] === 200;
    }
}
